#!/usr/bin/python

import json
import os
import torch
import esm
import time
from datetime import timedelta
from datetime import datetime
import numpy as np
import csv

dir_path = os.path.dirname(os.path.realpath(__file__)) + "/"
print(dir_path)

# Check the ESM settings to determine which input file to process
settings_file = open(dir_path + "esm_settings.json")
settings = json.load(settings_file)['esm']

print(settings)
with open(settings['input_filename']) as input_file:
	data = json.load(input_file)

# Load ESM-1b model
# The alphabet is based on the 
model, alphabet = esm.pretrained.esm1b_t33_650M_UR50S()
batch_converter = alphabet.get_batch_converter()

ch_labels, batch_strs, batch_tokens = batch_converter(data)

# Extract per-residue representations (on CPU)
with torch.no_grad():
    results = model(batch_tokens, repr_layers=[33], return_contacts=True)
token_representations = results["representations"][33]

# Generate per-sequence representations via averaging
# NOTE: token 0 is always a beginning-of-sequence token, so the first residue is token 1.
sequence_representations = []
for i, (_, seq) in enumerate(data):
    sequence_representations.append(token_representations[i, 1 : len(seq) + 1].mean(0))

matrices = []
for tensor in sequence_representations:
    matrices.append(tensor.numpy())

np.savez(settings['output_filename'] + ".npz", matrices)